import os

import MySQLdb

from jtalks.util.Logger import Logger


class DB:
    """
      A class to work with DB (upload, download, search). Note, that for all the operations we need a connection to
     database which created by connect_to_database method. To connection need name of environments, which contained config file with properties to database.
    """
    env = None
    config = None
    dbHost = None
    dbUser = None
    dbPass = None
    dbName = None
    cursor = None
    connection = None
    jcName = None
    jcDescription = None
    jcUrl = None
    jcNotify = None
    poulpeAdminPass = None
    logger = Logger("DB")

    def __init__(self, dbsettings, scriptsettings):
        self.dbHost = dbsettings.host
        self.dbUser = dbsettings.user
        self.dbPass = dbsettings.password
        self.dbName = dbsettings.name
        self.jcName = scriptsettings.props('forum_name')
        self.jcDescription = scriptsettings.props('forum_description')
        self.jcUrl = scriptsettings.props('forum_url')
        self.jcNotify = scriptsettings.props('forum_notify')
        self.poulpeAdminPass = scriptsettings.props('forum_poulpe_admin_pass')

    def connect_to_database(self):
        """
          Create connection to database.
        """
        self.connection = MySQLdb.connect(host=self.dbHost, user=self.dbUser, passwd=self.dbPass)
        self.cursor = self.connection.cursor()

    def close_connection(self):
        """
          Close connection to database.
        """
        self.connection.close()

    def recreate_database(self):
        """
          Method removed database (whith name, which contains in dbName), and create new(empty)
          database whita same name.
        """
        self.logger.info("Recreating database {0}", self.dbName)
        self.cursor.execute('DROP DATABASE IF EXISTS ' + self.dbName)
        self.cursor.execute('CREATE DATABASE ' + self.dbName)


    def restore_database_from_file(self, backupPath):
        """
          This method get path to filename (.sql) with backup of database. And restoring backup to database whith name,
          which contains in dbName.
        """
        self.logger.info("Loading a db dump from [{0}]", backupPath)
        self.recreate_database()
        self.fix_definer_in_backup(backupPath)
        os.popen(
            'mysql -u ' + self.dbUser + ' --password=' + self.dbPass + ' ' + self.dbName + ' < ' + backupPath).read()
        self.logger.info("Database restored: environment=[{0}]".format(self.env))


    def fix_definer_in_backup(self, backupPath):
        """
         For MySQL.
         If database contains VIEWS then created backup of database (by mysqldump) file contains lines with permissions
         (to user from original database). When we restoring database on another server (the server that doesn't have
         that user) we get an ERROR. To fix this problem, before restore we need remove all entries
         (with DEFINER='{username}') and the view will be created and will be invoked by the user that created the DB.
        """
        os.popen("sed -i 's/DEFINER=[^*]*\*/\*/g' " + backupPath).read()
        self.logger.info("Database definer fixed: environment=[{0}]".format(self.env))

    def update_properties_to_preprod(self):
        """
         This method update application properties in database. Need to call after restore.
        """
        self.cursor.execute('USE ' + self.dbName)
        self.logger.info("Changing forum name to [{0}]", self.jcName)
        self.cursor.execute('UPDATE COMPONENTS SET NAME="' + self.jcName + '", DESCRIPTION="' + self.jcDescription
                            + '" where COMPONENT_TYPE="FORUM"')
        self.logger.info("Switching off mail notifications")
        self.cursor.execute('UPDATE PROPERTIES SET VALUE="' + self.jcUrl + '" where NAME="jcommune.url_address"')
        self.cursor.execute('UPDATE USERS SET PASSWORD=MD5(' + self.poulpeAdminPass + ') WHERE USERNAME="admin"')
        self.connection.commit()
    
   
  



import os
import re
import urllib
import sgmllib
import shutil

from jtalks.util.Logger import Logger


class JtalksArtifacts:
    logger = Logger('JtalksArtifacts')

    def __init__(self, repo='builds'):
        self.repo = repo

    def download_war(self, project, build):
        gav = Gav(project + '-web-view', 'org.jtalks.' + project, version='', extension='')
        nexus = Nexus()
        version_page_url = gav.to_url(nexus.nexus_url, self.repo)
        version = NexusPageWithVersions().parse(version_page_url).version(build)
        gav.version = version
        gav.extension = 'war'
        nexus.download(self.repo, gav, project + '.war')
        return gav, project + '.war'

    def download_plugins(self, project, version, artifact_ids=()):
        """ -> [str] """
        files = []
        for plugin in artifact_ids:
            gav, filename = self.download_plugin(project, version, plugin)
            files.append(filename)
        return files

    def download_plugin(self, project, version, artifact_id):
        gav = Gav(artifact_id, 'org.jtalks.' + project, version)
        tofile_path = artifact_id + '.' + gav.extension
        Nexus().download(self.repo, gav, tofile_path)
        return gav, tofile_path

    def deploy_plugins(self, to_dir, plugin_files=[]):
        """
        Puts plugins to the config to the specified folder on the FS. Cleans previous plugins of there are any.
        :param str to_dir: directory to put the plugins to. Will be created if it's absent.
        :param [str] plugin_files: file names to put to the target dir
        """
        if not to_dir:
            if len(plugin_files) > 0:
                self.logger.warn('Plugin dir was not specified in env configs while there are plugins specified '
                                 'to be deployed: [{0}]. Skipping plugin deployment', ','.join(plugin_files))
            return

        if not os.path.exists(to_dir):
            self.logger.info('Plugin dir did not exist, creating: [{0}]', to_dir)
            os.makedirs(to_dir)
        for filename in os.listdir(to_dir):  # rm previous plugins
            if filename.endswith('.jar'):
                os.remove(os.path.join(to_dir, filename))
        for plugin in plugin_files:
            shutil.move(plugin, to_dir)


class Nexus:
    """
    A class to work with Nexus (upload, download, search). Note, that for all the operations we need a build number
    to be passed into the constructor. Other properties may or may not be required (some of them can be determined,
    like project name and version by pom.xml).
    """
    logger = Logger("Nexus")

    def __init__(self, nexus='http://repo.jtalks.org/content/repositories/'):
        self.nexus_url = nexus

    def download(self, repo, gav, tofile_path):
        """
        :param repo - str
        :param gav - Gav
        :param tofile_path str
        """
        url = self.nexus_url + repo + '/' + gav.to_repo_path()
        self.logger.info('Downloading artifact: [{0}]', url)
        urllib.urlretrieve(url, tofile_path)


class Gav:
    def __init__(self, artifact_id, group_id, version, classifier='', extension='jar'):
        self.artifact_id = artifact_id
        self.group_id = group_id
        self.version = version
        self.classifier = classifier
        self.extension = extension

    def to_str(self):
        return '{0}:{1}:{2}:{3}:{4}' \
            .format(self.group_id, self.artifact_id, self.version, self.classifier, self.extension)

    def to_repo_path(self):
        path_pattern = '{0}/{1}'
        if self.version:
            path_pattern += '/{2}/{1}-{2}'
        if self.version and self.classifier:
            path_pattern += '-{3}'
        if self.version and self.extension:
            path_pattern += '.{4}'
        return path_pattern \
            .format(self.group_id.replace('.', '/'), self.artifact_id, self.version, self.classifier, self.extension)

    def to_url(self, nexus_url, repo):
        if not nexus_url.endswith('/'):
            nexus_url += '/'
        return nexus_url + repo + '/' + self.to_repo_path()


class NexusPageWithVersions(sgmllib.SGMLParser):
    """ Can parse full versions of the artifacts by build number. """
    hyperlinks = []

    def parse(self, url_with_versions):
        feed = urllib.urlopen(url_with_versions)
        page_content = feed.read()
        feed.close()

        self.feed(page_content)
        self.close()
        return self

    def start_a(self, attributes):
        for name, link in attributes:
            if name == "href":
                self.hyperlinks.append(link)

    def version(self, build_number):
        for link in self.hyperlinks:
            match = re.search('(http://.*/)([0-9]+\.[0-9]+\.[0-9]+\.\w+)(.*)', link)
            if match:
                version = match.group(2)
                if '.{0}.'.format(build_number) in version:
                    return version
        raise BuildNotFoundException(
            "Couldn't find a build number {0}. Here are all the links: {1}".format(build_number, self.hyperlinks))


class BuildNotFoundException(Exception):
    pass



import os

from jtalks.ScriptSettings import ScriptSettings
from jtalks.util.Logger import Logger


class EnvList:
    """ Can list all the envs and apps. """
    logger = Logger("EnvList")

    def __init__(self, global_configs_dir):
        """ :param str global_configs_dir: the environments/ dir """
        self.global_configs_dir = global_configs_dir

    def list_envs(self):
        self.logger.info('[%s]' % ', '.join(map(str, self.get_list_of_envs())))

    def get_list_of_envs(self):
        envs = os.listdir(self.global_configs_dir)  # dir contains folders with envs configuration
        global_configuration_file = 'global-configuration.cfg'
        if global_configuration_file in envs:
            envs.remove(global_configuration_file)
        return envs

    def list_projects(self, env):
        projects = os.listdir(os.path.join(self.global_configs_dir, env))
        if ScriptSettings.ENV_CONFIG_FILE_NAME in projects:
            projects.remove(ScriptSettings.ENV_CONFIG_FILE_NAME)
        return [project.replace(".cfg", "") for project in projects if project.endswith(".cfg")]

    def print_envs_with_projects(self):
        for env in self.get_list_of_envs():
            print env
            for project in self.list_projects(env):
                print "  |_" + project



import os
import unittest
import shutil

from jtalks.Tomcat import Tomcat, TomcatNotFoundException


class TomcatTest(unittest.TestCase):
    def setUp(self):
        os.mkdir('test_tomcat')

    def tearDown(self):
        shutil.rmtree('test_tomcat')

    def test_move_war_to_webapps_should_actually_moves_it(self):
        # given
        tmpfile = file('test_tomcat/tomcat-test-project.tmp', 'w')
        os.mkdir('test_tomcat/webapps')
        # when
        dst_filename = Tomcat('test_tomcat').move_to_webapps(tmpfile.name, 'tomcat-test-project')
        # then
        self.assertEqual('test_tomcat/webapps/tomcat-test-project.war', dst_filename)
        self.assertTrue(os.path.exists(dst_filename))

    def test_move_war_deletes_prev_app_dir(self):
        # given
        tmpdir = os.path.join('test_tomcat', 'webapps', 'tomcat-test-project', 'tmpdir')
        os.makedirs(tmpdir)
        tmpfile = file('test_tomcat/tomcat-test-project', 'w')
        # when
        Tomcat('test_tomcat').move_to_webapps(tmpfile.name, 'tomcat-test-project')
        # then:
        self.assertFalse(os.path.exists(tmpdir), 'Tomcat did not remove previous app folder from webapps')

    def test_move_war_to_webapps_raises_if_tomcat_location_is_wrong(self):
        tomcat = Tomcat('./')
        self.assertRaises(TomcatNotFoundException, tomcat.move_to_webapps, 'src_filepath', 'appname')

    def test_start(self):
        os.makedirs('test_tomcat/bin')
        file('test_tomcat/bin/startup.sh', 'w').write('echo test > test_tomcat/test; sleep 5 &')
        Tomcat('test_tomcat').start()
        self.assertEqual('test\n', file('test_tomcat/test').readline())



from optparse import Values
import unittest

from jtalks.ApplicationContext import ApplicationContext
from jtalks.Tomcat import Tomcat
from jtalks.ScriptSettings import ScriptSettings


class ApplicationContextTest(unittest.TestCase):
    sut = ApplicationContext(ScriptSettings(Values({'project': 'project', 'env': 'system-test', 'build': 510,
                                                    'grab_envs': 'false', 'sanity_test_timeout_sec': 120}),
                                            workdir='.jtalks'))

    def test_nexus_is_created(self):
        nexus = self.sut.old_nexus()
        self.assertEquals(nexus.build_number, 510)

    def test_tomcat_is_created(self):
        tomcat = self.sut.tomcat()
        # can't use assertIsInstance() because on CentOS we use old mock lib
        self.assertTrue(isinstance(tomcat, Tomcat))



import os
import shutil
from git import GitCommandError
import git.repo as repo
from jtalks.util.Logger import Logger


class EnvironmentConfigGrabber:
    logger = Logger('EnvironmentConfigGrabber')

    def __init__(self, env_configs_root, temp_dir):
        self.env_configs_root = env_configs_root
        self.temp_dir = temp_dir
        self.clone_repo_to = os.path.join(temp_dir, 'environments')
        self.grabbed_configs_location = os.path.join(self.clone_repo_to, 'configs')

    def grab_jtalks_configs(self):
        try:
            self.__remove_previous_git_folder__()
            self.__create_jtalks_temp_dir__()
            self.logger.info('Cloning environments to {0}', self.clone_repo_to)
            repo.Repo.clone_from('git@jtalks.org:environments', self.clone_repo_to)
            self.__copy_grabbed_configs_into_work_dir__()
        except GitCommandError as e:
            self.logger.warn(
                "You don't have access to JTalks repo with environment configs. You may want to use your own "
                + "local environment. Create a folder with env name in configs directory. If you think you "
                + "need access to JTalks internal envs (including UAT, DEV, PREPROD, PROD envs), "
                + "you should write a request to project@jtalks.org.")
            raise e

    def __remove_previous_git_folder__(self):
        try:
            if os.path.exists(self.clone_repo_to):
                self.logger.info("Removing {0} directory if it was there", self.clone_repo_to)
                shutil.rmtree(self.clone_repo_to)
        except OSError as e:
            if e.errno is not 2:  # No such file or directory
                self.logger.warn(e.message)

    def __copy_grabbed_configs_into_work_dir__(self):
        grabbed_dirs_and_files = os.listdir(self.grabbed_configs_location)
        for next_grabbed_file_or_dir in grabbed_dirs_and_files:
            destination_file = os.path.join(self.env_configs_root, next_grabbed_file_or_dir)
            self.logger.info('Creating dir [{0}]', os.path.abspath(destination_file))
            if os.path.exists(destination_file):
                self.__delete_file_or_dir__(destination_file)
            shutil.move(os.path.join(self.grabbed_configs_location, next_grabbed_file_or_dir), destination_file)

    def __delete_file_or_dir__(self, destination_file):
        if os.path.isfile(destination_file):
            os.remove(destination_file)
        else:
            shutil.rmtree(destination_file)

    def __create_jtalks_temp_dir__(self):
        if not os.path.exists(self.temp_dir):
            os.mkdir(self.temp_dir)



import os

import MySQLdb

from jtalks.util.Logger import Logger


class DbOperations:
    """ Base class for working with database """
    logger = Logger("DbBase")

    def __init__(self, dbsettings):
        """ :param jtalks.ScripSettings.DbSettings dbsettings: settings """
        self.dbsettings = dbsettings

    def connect_to_database(self):
        """ Connects to the specified database """
        self.logger.info("Connecting to [{0}] with user [{1}]", self.dbsettings.host, self.dbsettings.user)
        self.connection = MySQLdb.connect(host=self.dbsettings.host, user=self.dbsettings.user,
                                          passwd=self.dbsettings.password)
        self.cursor = self.connection.cursor()

    def close_connection(self):
        """ Closes open connection to the database """
        self.connection.close()

    def recreate_database(self):
        """ Deletes the specified database and create it again """
        self.logger.info("Dropping and creating database from scratch: [{0}]", self.dbsettings.name)
        self.cursor.execute('DROP DATABASE IF EXISTS ' + self.dbsettings.name)
        self.cursor.execute('CREATE DATABASE ' + self.dbsettings.name)

    def backup_database(self, backup_path):
        """ Backups database to given backupPath """
        self.logger.info("Backing up database [{0}] to [{1}] using user [{2}]",
                         self.dbsettings.name, backup_path, self.dbsettings.user)
        dump_command = "mysqldump -u'{0}'".format(self.dbsettings.user)
        if self.dbsettings.password:
            dump_command += "-p'{0}'".format(self.dbsettings.password)
        dump_command += "'{0}' > '{1}/{0}.sql'".format(self.dbsettings.name, backup_path)
        os.popen(dump_command.format(self.dbsettings.user, self.dbsettings.password,
                                     self.dbsettings.name, backup_path)).read()
        self.logger.info("Database backed up [{0}]".format(self.dbsettings.name))

    def restore_database_from_file(self, backupPath):
        """ Restores database from file specified in backupPath """
        self.logger.info("Loading a db dump from [{0}/{1}.sql]", backupPath, self.dbsettings.name)
        self.recreate_database()
        os.popen("mysql -u'{0}' -p'{1}' '{2}' < '{3}/{2}.sql'"
                 .format(self.dbsettings.user, self.dbsettings.password, self.dbsettings.name, backupPath)).read()
        self.logger.info("Database restored [{0}]".format(self.dbsettings.name))






import os
from os import path
import unittest
from datetime import datetime
import shutil

from jtalks.backup.Backuper import Backuper


class BackuperTest(unittest.TestCase):
    def setUp(self):
        os.mkdir('backup')
        os.mkdir('to_backup')

    def tearDown(self):
        shutil.rmtree('backup')
        shutil.rmtree('to_backup')

    def test_backup_creates_folder_to_keep_backups(self):
        Backuper('./backup/', None)
        self.assertTrue(path.exists('backup/' + self.get_now_formatted_as_backup()))

    def test_backup_folder_actually_backs_up(self):
        os.mkdir('to_backup/test')
        backuper = Backuper('./backup/', None)
        Backuper('./backup/', None).back_up_dir('./to_backup')
        self.assertTrue(path.exists(path.join(backuper.backup_folder, 'to_backup/test')))

    def get_now_formatted_as_backup(self):
        return datetime.now().strftime("%Y_%m_%dT%H_%M_%S")



import os
from unittest import TestCase

from jtalks import OldNexus


class OldNexusTest(TestCase):
    NEXUS_BASE_URL = 'http://repo.jtalks.org/content/repositories/deployment-pipeline/deployment-pipeline'

    def test_get_war_url(self):
        nexus = OldNexus.Nexus(build_number=2600)
        self.assertEqual(nexus.get_war_url('jcommune', 2600), self.url('2.12.2600.e148537'))

    def test_download_war(self):
        nexus = OldNexus.Nexus(build_number=2600)
        nexus.download_war('jcommune')
        self.assertTrue(os.path.exists('jcommune.war'))
        self.assertEqual(os.stat('jcommune.war').st_size, 38954194)
        os.remove('jcommune.war')

    def url(self, version):
        return self.NEXUS_BASE_URL + '/jcommune/' + version + '/jcommune-' + version + '.war'



from optparse import Values
import os
from os.path import join
import unittest
import shutil

from jtalks.ScriptSettings import ScriptSettings, RequiredPropertyNotFoundException


class ScriptSettingsTest(unittest.TestCase):
    tmp_dir = 'ScripSettingsTest-TmpDir'

    def setUp(self):
        os.mkdir(self.tmp_dir)
        os.makedirs(join(self.tmp_dir, ScriptSettings.ENVS_DIR_NAME, 'env'))
        os.mkdir(join(self.tmp_dir, ScriptSettings.TEMP_DIR_NAME))
        os.mkdir(join(self.tmp_dir, ScriptSettings.BACKUPS_DIR_NAME))

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_global_prop_is_overriden_by_env_prop_which_is_overriden_by_project_prop(self):
        self.write_global_config('[section]\nglobal=global\nenv=global\nproject=global')
        self.write_env_config('[section]\nenv=env\nproject=env')
        self.write_project_config('[section]\nproject=project')
        self.assertEquals('global', self.sut().props['section_global'])
        self.assertEquals('env', self.sut().props['section_env'])
        self.assertEquals('project', self.sut().props['section_project'])

    def test_props_with_special_symbols(self):
        self.write_global_config('[sec-dash.dot]\nopt-dash.dot=val-dash.dot')
        self.assertEqual('val-dash.dot', self.sut().props['sec-dash.dot_opt-dash.dot'])

    def test_placeholder_replaced(self):
        self.write_global_config('[sec]\nopt1=${sec_opt2}\nopt2=${sec_opt3}\nopt3=value')
        self.write_env_config('[sec1]\nproj=${project}\nenv=${env}\nopt3=value')
        self.assertEqual('value', self.sut().props['sec_opt1'])
        self.assertEqual('project', self.sut().props['sec1_proj'])
        self.assertEqual('env', self.sut().props['sec1_env'])

    def test_plugins_section_is_split_into_separate_plugins(self):
        self.write_global_config('[app]\nplugins=plugin1,plugin-2,plugin 3')
        self.assertEqual(['plugin1', 'plugin-2', 'plugin 3'], self.sut().get_plugins())
        self.assertNotEqual(('plugin1', 'plugin-2', 'plugin 3'), self.sut().get_plugins())

        self.write_global_config('[app]\nplugins=plugin1')
        self.assertEqual(['plugin1'], self.sut().get_plugins())
        self.assertNotEqual('plugin1', self.sut().get_plugins())

        self.write_global_config('[app]\nplugins=')
        self.assertEqual(0, len(self.sut().get_plugins()))

        self.write_global_config('[app]\n')
        self.assertEqual(0, len(self.sut().get_plugins()))

    def test_settings_file_mapping(self):
        self.write_global_config('[app-files]\nfile1=/file1\nfile-2=/file2\nfile 3=${project}.xml')
        mapping = self.sut().get_app_file_mapping()
        self.assertEqual('/file1', mapping['file1'])
        self.assertEqual('/file2', mapping['file-2'])
        self.assertEqual('project.xml', mapping['file 3'])

    def test_files_are_deployed(self):
        sut = self.sut()
        file(join(sut.global_configs_dir, 'global-file.xml'), 'w').write('Hello, ${project}')
        file(join(sut.global_configs_dir, 'env-file.properties'), 'w').write('This must be overriden by env file')
        file(join(sut.env_configs_dir, 'env-file.properties'), 'w').write('Hello, ${env}')
        self.write_global_config('[tmp]\ndir=' + self.tmp_dir +
                                 '\n[app-files]'
                                 '\nglobal-file.xml=${tmp_dir}/file.xml'
                                 '\nenv-file.properties=${tmp_dir}/creates_folder/file.properties')
        self.sut().deploy_configs()
        self.assertEqual('Hello, project', file(join(self.tmp_dir, 'file.xml')).readline())
        self.assertEqual('Hello, env', file(join(self.tmp_dir, 'creates_folder', 'file.properties')).readline())

    def test_get_db_settings(self):
        self.write_env_config('[app]\ndb_user=u\ndb_password=pass\ndb_name=name\ndb_host=local\ndb_port=22')
        db_settings = self.sut().get_db_settings()
        self.assertEqual('u', db_settings.user)
        self.assertEqual('pass', db_settings.password)
        self.assertEqual('name', db_settings.name)
        self.assertEqual('local', db_settings.host)
        self.assertEqual(22, db_settings.port)

    def test_get_db_settings_for_optional_fields(self):
        self.write_env_config('[app]\ndb_user=u\ndb_name=name\ndb_host=local')
        db_settings = self.sut().get_db_settings()
        self.assertEqual('', db_settings.password)
        self.assertEqual(3306, db_settings.port)

    def test_get_db_settings_if_required_fields_not_found(self):
        self.write_env_config('[app]\ndb_user=u\ndb_name=name')
        self.assertRaises(RequiredPropertyNotFoundException, self.sut().get_db_settings)

        self.write_env_config('[app]\ndb_user=u\ndb_host=local')
        self.assertRaises(RequiredPropertyNotFoundException, self.sut().get_db_settings)

        self.write_env_config('[app]\ndb_name=name\ndb_host=local')
        self.assertRaises(RequiredPropertyNotFoundException, self.sut().get_db_settings)

    def write_global_config(self, text):
        file(join(self.tmp_dir, ScriptSettings.ENVS_DIR_NAME, ScriptSettings.GLOBAL_ENV_CONFIG_FILE_NAME), 'w')\
            .write(text)

    def write_env_config(self, text, env='env'):
        env_dir = join(self.tmp_dir, ScriptSettings.ENVS_DIR_NAME, env)
        if not os.path.exists(env_dir):
            os.makedirs(env_dir)
        file(join(env_dir, ScriptSettings.ENV_CONFIG_FILE_NAME), 'w').write(text)

    def write_project_config(self, text, env='env', project='project'):
        env_dir = join(self.tmp_dir, ScriptSettings.ENVS_DIR_NAME, env)
        if not os.path.exists(env_dir):
            os.makedirs(env_dir)
        file(join(env_dir, project + '.cfg'), 'w').write(text)

    def sut(self, options_as_dict={}):
        """
        :param dict options_as_dict: options to pass to script settings
        :return: ScriptSettings
        """
        merged = dict(options_as_dict.items() + {'project': 'project', 'env': 'env', 'build': '0', 'grab_envs': 'false',
                                                 'sanity_test_timeout_sec': 120}.items())
        return ScriptSettings(Values(merged), workdir=self.tmp_dir)



from time import sleep
import datetime

import requests
from requests.exceptions import ConnectionError

from jtalks.sanity.SanityCheckFailedException import SanityCheckFailedException
from jtalks.util.Logger import Logger


class SanityTest:
    """
      Tests that application was deployed correctly without errors. For these purposes it opens some pages and determines
      whether they return HTML. If let's say they return HTTP 500, then the test failed. It has to break CI builds.
    """
    HOST = "127.0.0.1"
    logger = Logger("SanityTest")
    port = None
    app_name = None

    def __init__(self, tomcat_port, app_name, sanity_test_timeout_sec=120, sleep_sec=30):
        """
         @param tomcat_port - an HTTP port to access the web server where application is
         @param sleep_sec - the amount of time tests ignore error responses as deployment failure. This is needed because
          first tomcat may not start quickly and therefore the response Connection Refused will be immediate. Thus when we
          send requests, first we should treat error messages as possible responses. After this sleep time error responses
          are considered as failed deployment.
        """
        self.port = int(tomcat_port)
        self.app_name = app_name
        self.sanity_test_timeout_sec = sanity_test_timeout_sec
        self.sleep_sec = sleep_sec
        if app_name == "ROOT":
            self.app_name = ""

    def check_app_started_correctly(self):
        request_address = "http://{0}:{1}/{2}".format(self.HOST, self.port, self.app_name)
        tests_sleep_end = datetime.datetime.now() + datetime.timedelta(seconds=self.sleep_sec)
        response = None
        attempt_counter = 0
        while tests_sleep_end > datetime.datetime.now():
            attempt_counter += 1
            self.logger.info(
                "[Attempt #{0}] Running sanity tests to check whether application started correctly and responds back..",
                attempt_counter)
            self.logger.info("[Attempt #{0}] Connecting to {1}", attempt_counter, request_address)
            try:
                response = requests.get(request_address, timeout=self.sanity_test_timeout_sec)
            except ConnectionError:
                self.logger.info("Sleeping for 5 sec..")
                sleep(5)  # so that we don't connect too often
                continue
            if response.status_code in [200, 201]:
                break
            else:
                self.logger.info("Error response, got {0} HTTP status. App server might still be booting.",
                                 response.status_code)
        if not response or response.status_code not in [200, 201]:
            self.logger.error('After {0} no successful response was received from the app. Finishing by timeout {1}',
                              attempt_counter, self.sanity_test_timeout_sec)
            if response:
                self.logger.error("Last time while accessing main page, app answered with error: [{0} {1} {2}]",
                                  response.status_code, response.reason, response.text)
            else:
                self.logger.error("App Server did not even get up!")
            raise SanityCheckFailedException("Sanity check failed")
        self.logger.info("Sanity check passed: [{0} {1}]", response.status_code, response.reason)



from ConfigParser import ConfigParser
import os

from jtalks import __version__
from jtalks.util.Logger import Logger


class ScriptSettings:
    logger = Logger("ScriptSettings")
    TEMP_DIR_NAME = 'temp'
    BACKUPS_DIR_NAME = 'backups'
    ENVS_DIR_NAME = 'environments'
    GLOBAL_ENV_CONFIG_FILE_NAME = 'global-configuration.cfg'
    ENV_CONFIG_FILE_NAME = 'environment-configuration.cfg'

    def __init__(self, options_passed_to_script, workdir=os.path.expanduser('~/.jtalks')):
        """ :param optparse.Values options_passed_to_script: thins that are passed with -f or --flags """
        self.env = options_passed_to_script.env
        self.build = options_passed_to_script.build
        self.project = options_passed_to_script.project
        self.grab_envs = options_passed_to_script.grab_envs
        self.sanity_test_timeout_sec = options_passed_to_script.sanity_test_timeout_sec
        self.package_version = __version__

        self.work_dir = workdir
        self.temp_dir = os.path.join(self.work_dir, self.TEMP_DIR_NAME)
        self.backups_dir = os.path.join(self.work_dir, self.BACKUPS_DIR_NAME)
        self.global_configs_dir = os.path.join(self.work_dir, self.ENVS_DIR_NAME)
        self.env_configs_dir = os.path.join(self.global_configs_dir, self.env)

        self.global_config_path = os.path.join(self.global_configs_dir, self.GLOBAL_ENV_CONFIG_FILE_NAME)
        self.env_config_path = os.path.join(self.env_configs_dir, self.ENV_CONFIG_FILE_NAME)
        self.project_config_path = os.path.join(self.env_configs_dir, self.project + '.cfg')

        self.props = self._read_properties()

    def log_settings(self):
        self.logger.info('Script Settings: project=[{0}], env=[{1}], build number=[{2}], sanity test timeout=[{3}], '
                         'package version=[{4}]',
                         self.project, self.env, self.build, self.sanity_test_timeout_sec, self.package_version)
        self.logger.info("Environment configuration: [{0}]", self.env_configs_dir)

    def create_work_dir_if_absent(self):
        self._create_dir_if_absent(self.work_dir)
        self._create_dir_if_absent(self.env_configs_dir)
        self._create_dir_if_absent(self.backups_dir)

    def _create_dir_if_absent(self, directory):
        if not os.path.exists(directory):
            self.logger.info("Creating directory [{0}]", directory)
            os.makedirs(directory)

    def get_tomcat_port(self):
        return int(self.props.get('tomcat_http_port', 0))

    def get_tomcat_location(self):
        """ Gets value of the tomcat home from [project].cfg file related to particular env and project """
        return self.props.get('tomcat_location', None)

    def get_app_final_name(self):
        """ Gets the name of the application to be deployed (even if it's Poulpe, it can be deployed as ROOT.war). """
        return self.props.get('app_final_name', self.project)

    def get_plugins(self):
        if 'app_plugins' in self.props and self.props['app_plugins']:
            return self.props['app_plugins'].split(',')
        else:
            return []

    def get_plugins_dir(self):
        if 'app_plugins_dir' in self.props and self.props['app_plugins_dir']:
            return self.props['app_plugins_dir']
        return None

    def get_app_file_mapping(self):
        """
        :return dict: mapping of the src file name (located either in environments/ or in environments/env folder)
            without folder part and the destination file path (where to put those files on the server)
        """
        file_mapping = {}
        app_files_section = 'app-files_'
        for key in self.props.keys():
            if key.startswith(app_files_section):
                src_filename = key.replace(app_files_section, '')
                dst_filepath = self.props[key]
                file_mapping[src_filename] = dst_filepath
        return file_mapping

    def get_db_settings(self):
        """ :return DbSettings: db settings """
        return DbSettings.build(self.props)

    def deploy_configs(self):
        mapping = self.get_app_file_mapping()
        for key in mapping:
            src_filepath = os.path.join(self.env_configs_dir, key)
            if not os.path.exists(src_filepath):
                src_filepath = os.path.join(self.global_configs_dir, key)
            if not os.path.exists(src_filepath):
                self.logger.info('File {0} did not exist, skipping its deployment', key)
                continue
            dst_filepath = mapping[key]
            self.logger.info('Putting file [{0}]', dst_filepath)
            if not os.path.exists(os.path.dirname(dst_filepath)):
                self.logger.info('Creating dir [{0}] to place the file', os.path.dirname(dst_filepath))
                os.makedirs(os.path.dirname(dst_filepath))
            dst_file = file(dst_filepath, 'w')
            for line in open(src_filepath).readlines():
                dst_file.write(self._resolve_placeholder(self.props, line))
            dst_file.close()

    def _read_properties(self):
        """
        Reads props from global, env and project configs, then returns them as map with `section_option=value`.
        Values may contain placeholders referencing other options and special options like `${project}` & `${env}`
        """
        configs = [os.path.abspath(self.global_config_path), os.path.abspath(self.env_config_path),
                   os.path.abspath(self.project_config_path)]
        props = {}
        config = ConfigParser()
        for config_file in configs:
            abs_path = os.path.abspath(config_file)
            if os.path.exists(abs_path):
                self.logger.info('Reading config file [{0}]', abs_path)
                config.read(abs_path)
            else:
                self.logger.info('Could not read config file as it does not exist: [{0}]', abs_path)
        for section in config.sections():
            for option in config.options(section):
                props[section + '_' + option] = config.get(section, option)
        with_replaced_placeholders = {}
        for key in props.keys():
            with_replaced_placeholders[key] = self._resolve_placeholder(props, props[key])
        return with_replaced_placeholders

    def _resolve_placeholder(self, props, value, n_of_trial=0):
        if n_of_trial > 10:
            self.logger.warn('Could not resolve all placeholders in property value: {0}. Leaving it as is.', value)
            return value
        value = value.replace("${env}", self.env).replace("${project}", self.project)
        if value.find('${') != -1:
            for key in props.keys():
                value = value.replace('${' + key + '}', props[key])
        if value.find('${') != -1:
            value = self._resolve_placeholder(props, value, n_of_trial+1)
        return value


class DbSettings:
    REQUIRED_PROPS = ['app_db_name', 'app_db_user', 'app_db_host']

    def __init__(self, host, name, user, password, port=3306):
        self.password = password
        self.user = user
        self.port = port
        self.name = name
        self.host = host

    @staticmethod
    def build(props):
        """ :return DbSettings: db settings """
        for propname in DbSettings.REQUIRED_PROPS:
            if propname not in props.keys():
                raise RequiredPropertyNotFoundException(
                    'Property {0} was not found. All these properties are required: {1}'.format(
                        propname, ', '.join(DbSettings.REQUIRED_PROPS)))
        port = 3306
        password = ''
        if 'app_db_port' in props:
            port = int(props['app_db_port'])
        if 'app_db_password' in props:
            password = props['app_db_password']
        return DbSettings(props['app_db_host'], props['app_db_name'], props['app_db_user'], password, port)


class RequiredPropertyNotFoundException(Exception):
    pass



from jtalks.DeployCommand import DeployCommand
from jtalks.OldNexus import Nexus as OldNexus
import jtalks.Nexus as NewNexus
from jtalks.DB import DB
from jtalks.SSH import SSH
from jtalks.Tomcat import Tomcat
from jtalks.backup.Backuper import Backuper
from jtalks.db.DbOperations import DbOperations
from jtalks.db.LoadDbFromBackup import LoadDbFromBackup
from jtalks.sanity.SanityTest import SanityTest
from jtalks.util.EnvList import EnvList
from jtalks.util.EnvironmentConfigGrabber import EnvironmentConfigGrabber


class ApplicationContext:
    """
      Carries similar ideas to Spring's AppContext - it builds all the objects. This IoC is manual though, the objects
      are all constructed manually in Python code.
    """

    def __init__(self, script_settings):
        """ :param jtalks.ScriptSettings.ScriptSettings script_settings: settings """
        self.script_settings = script_settings
        self.script_settings.create_work_dir_if_absent()
        self.script_settings.log_settings()

    def old_nexus(self):
        return OldNexus(build_number=self.script_settings.build)

    def jtalks_artifacts(self):
        return NewNexus.JtalksArtifacts()

    def tomcat(self):
        return Tomcat(self.script_settings.get_tomcat_location())

    def backuper(self):
        return Backuper(self.script_settings.backups_dir, self.db_operations())

    def db_operations(self):
        return DbOperations(self.script_settings.get_db_settings())

    def db_settings(self):
        return self.script_settings.get_db_settings()

    def deploy_command(self):
        return DeployCommand(
            self.jtalks_artifacts(), self.old_nexus(), self.tomcat(), self.sanity_test(),
            self.backuper(), self.script_settings)

    def environment_config_grabber(self):
        return EnvironmentConfigGrabber(self.script_settings.global_configs_dir, self.script_settings.temp_dir)

    def sanity_test(self):
        http_port = self.script_settings.get_tomcat_port()
        return SanityTest(tomcat_port=http_port, app_name=self.script_settings.get_app_final_name(),
                          sanity_test_timeout_sec=self.script_settings.sanity_test_timeout_sec)

    def load_db_from_backup(self):
        return LoadDbFromBackup(self.db(), self.ssh())

    def ssh(self):
        return SSH(self.script_settings)

    def db(self):
        return DB(self.script_settings)

    def script_settings(self):
        return self.script_settings

    def env_list(self):
        return EnvList(self.script_settings.global_configs_dir)



import os
import unittest
import shutil

from jtalks.util.EnvList import EnvList


class EnvListTest(unittest.TestCase):
    tmp_dir = 'EnvListTestResources'
    global_config_dir = 'EnvListTestResources/environments'
    env_dir = os.path.join(global_config_dir, 'env')

    def setUp(self):
        os.makedirs(self.env_dir)

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_all_envs_are_listed(self):
        self.assertEqual(len(EnvList(self.global_config_dir).get_list_of_envs()), 1)
        self.assertEqual(['env'], EnvList(self.global_config_dir).get_list_of_envs())

    def test_list_projects_for_env(self):
        file(os.path.join(self.env_dir, 'project.cfg'), 'w')
        projects = EnvList(self.global_config_dir).list_projects('env')
        self.assertEqual(['project'], projects)



import shutil
import subprocess
from subprocess import PIPE
import os

from jtalks.util.Logger import Logger


class Tomcat:
    """ Class for deploying and backing up Tomcat applications """

    logger = Logger("Tomcat")

    def __init__(self, tomcat_location):
        """ :param str tomcat_location: location of the tomcat root dir """
        self.tomcat_location = tomcat_location
        if self.tomcat_location and os.path.exists(tomcat_location):
            self.logger.info('Tomcat location: [{0}]', tomcat_location)
        else:
            self.logger.warn('Tomcat location was not set or it does not exist: [{0}]', tomcat_location)

    def stop(self):
        """ Stops the Tomcat server if it is running """
        if not os.path.exists(self.tomcat_location):
            raise TomcatNotFoundException('Could not found tomcat: [{0}]'.format(self.tomcat_location))
        stop_command = 'pkill -9 -f {0}'.format(os.path.abspath(self.tomcat_location))
        self.logger.info('Killing tomcat [{0}]', stop_command)
        # dunno why but return code always equals to SIGNAL (-9 in this case), didn't figure out how to
        # distinguish errors from this
        subprocess.call([stop_command], shell=True, stdout=PIPE, stderr=PIPE)

    def start(self):
        """ Starts the Tomcat server """
        if not os.path.exists(self.tomcat_location):
            raise TomcatNotFoundException('Could not found tomcat: [{0}]'.format(self.tomcat_location))
        startup_file = self.tomcat_location + "/bin/startup.sh"
        self.logger.info("Starting Tomcat [{0}]", startup_file)
        pipe = subprocess.Popen(['/bin/bash', startup_file], shell=False, stdout=PIPE, stderr=PIPE)
        out, err = pipe.communicate()
        if pipe.returncode != 0:
            error = 'Could not start Tomcat, return code: {0}'.format(pipe.returncode)
            self.logger.error(error)
            self.logger.error(out)
            self.logger.error(err)
            raise CouldNotStartTomcatException(error)

    def move_to_webapps(self, src_filepath, appname):
        """
        Moves application war-file to 'webapps' Tomcat sub-folder
        :param str src_filepath: to get artifact from
        :param str appname: the name of the webapp to be deployed
        """
        webapps_location = os.path.join(self.tomcat_location, 'webapps')
        final_app_location = os.path.join(webapps_location, appname)
        self.logger.info('Putting new war file to Tomcat: [{0}]', final_app_location)
        if not os.path.exists(webapps_location):
            error = 'Tomcat webapps folder was not found in [{0}], configuration must have been wrong. ' \
                    'Please configure correct Tomcat location. Current location contains: {1}' \
                .format(self.tomcat_location, os.listdir(self.tomcat_location))
            self.logger.error(error)
            raise TomcatNotFoundException(error)
        self._remove_previous_app(final_app_location)
        shutil.move(src_filepath, final_app_location + '.war')
        return final_app_location + '.war'

    def _remove_previous_app(self, app_location):
        if os.path.exists(app_location):
            self.logger.info("Removing previous app: [{0}]", app_location)
            shutil.rmtree(app_location)
        else:
            self.logger.info("Previous application was not found in [{0}], thus nothing to remove", app_location)

        war_location = app_location + ".war"
        if os.path.exists(war_location):
            self.logger.info("Removing previous war file: [{0}]", war_location)
            os.remove(war_location)


class TomcatNotFoundException(Exception):
    pass


class FileNotFoundException(Exception):
    pass


class CouldNotStartTomcatException(Exception):
    pass



from optparse import Values
from os import path
import os
import unittest

from jtalks.Tomcat import Tomcat
from jtalks.OldNexus import Nexus
from jtalks.Nexus import JtalksArtifacts
from jtalks.DeployCommand import DeployCommand
from jtalks.backup.Backuper import Backuper
from jtalks.db.DbOperations import DbOperations
from jtalks.sanity.SanityTest import SanityTest
from jtalks.ScriptSettings import ScriptSettings


class DeployCommandTest(unittest.TestCase):
    def test_deployment_from_old_nexus_repo(self):
        scriptsettings = self._scriptsettings(build=2789)
        try:
            self._deploy(scriptsettings)
        except:
            self._read_log_if_available('/home/jtalks/tomcat/logs/jcommune.log')
            self._read_log_if_available('/home/jtalks/tomcat/logs/catalina.out')
            raise

    def test_deployment_from_new_nexus_repo(self):
        scriptsettings = self._scriptsettings(build=6)
        try:
            self._deploy(scriptsettings)
        except:
            self._read_log_if_available('/home/jtalks/tomcat/logs/jcommune.log')
            self._read_log_if_available('/home/jtalks/tomcat/logs/catalina.out')
            raise

    def test_deploy_poulpe_from_old_nexus_repo(self):
        scriptsettings = self._scriptsettings(build=344, project='poulpe')
        try:
            self._deploy(scriptsettings)
        except:
            self._read_log_if_available('/home/jtalks/tomcat/logs/poulpe.log')
            self._read_log_if_available('/home/jtalks/tomcat/logs/catalina.out')
            raise

    def _deploy(self, scriptsettings):
        DeployCommand(
            JtalksArtifacts(), Nexus(scriptsettings.build), Tomcat(path.expanduser('~/tomcat')),
            SanityTest(scriptsettings.get_tomcat_port(), scriptsettings.project),
            Backuper('/home/jtalks', DbOperations(scriptsettings.get_db_settings())), scriptsettings
        ).deploy(scriptsettings.project, scriptsettings.build,
                 scriptsettings.get_app_final_name(), scriptsettings.get_plugins())

    def _scriptsettings(self, build, project='jcommune'):
        return ScriptSettings(Values({'env': 'system-test', 'project': project, 'build': build,
                                      'grab_envs': 'false', 'sanity_test_timeout_sec': 120}),
                              workdir='tests/.jtalks')

    def _read_log_if_available(self, log_file):
        if os.path.exists(log_file):
            print('Reading log file [{0}]'.format(log_file))
            f = file(log_file, 'r')
            for line in f.readlines():
                print(line)
        else:
            print('Log file {0} was not created, not logs to show'.format(log_file))



import os
from unittest import TestCase
import shutil

from jtalks.Nexus import Gav, Nexus, JtalksArtifacts, NexusPageWithVersions


class NexusTest(TestCase):
    def setUp(self):
        os.makedirs('NexusTestDir')

    def tearDown(self):
        shutil.rmtree('NexusTestDir')

    def test_gav_to_str(self):
        self.assertEqual('g:a:v::jar', Gav('a', 'g', 'v').to_str())
        self.assertEqual('g:a:v:c:jar', Gav('a', 'g', 'v', 'c').to_str())
        self.assertEqual('g:a:v:c:war', Gav('a', 'g', 'v', 'c', 'war').to_str())

    def test_gav_to_repo_path(self):
        self.assertEqual('g/a/v/a-v.jar', Gav('a', 'g', 'v').to_repo_path())
        self.assertEqual('g/a/v/a-v-c.jar', Gav('a', 'g', 'v', 'c').to_repo_path())
        self.assertEqual('g/a/v/a-v-c.war', Gav('a', 'g', 'v', 'c', 'war').to_repo_path())
        self.assertEqual('g/g/a/v/a-v-c.war', Gav('a', 'g.g', 'v', 'c', 'war').to_repo_path())
        self.assertEqual('g/g/a/v/a-v-c', Gav('a', 'g.g', 'v', 'c', '').to_repo_path())
        self.assertEqual('g/g/a/v/a-v', Gav('a', 'g.g', 'v', '', '').to_repo_path())
        self.assertEqual('g/g/a', Gav('a', 'g.g', '', '', '').to_repo_path())
        self.assertEqual('g/g/a', Gav('a', 'g.g', '', 'c', '').to_repo_path())
        self.assertEqual('g/g/a', Gav('a', 'g.g', '', '', 'war').to_repo_path())

    def test_gav_to_url(self):
        self.assertEqual('http://nexus/repo/g/a/v/a-v.jar', Gav('a', 'g', 'v').to_url('http://nexus', 'repo'))
        self.assertEqual('http://nexus/repo/g/a/v/a-v.jar', Gav('a', 'g', 'v').to_url('http://nexus/', 'repo'))

    def test_parse_version_from_url(self):
        versions_links = NexusPageWithVersions()
        versions_links.hyperlinks = ['http://n/a/b/c/3.0.5.0f5b2cf/', 'http://n/a/b/c/30.-0.6.0f5b2cf/',
                                     'http://n/a/b/c/30.0.7.0f5b2cf/', 'http://n/a/b/c/3.0.8/',
                                     'http://n/a/b/c/3.0.10.0f5b2cf/']
        self.assertEqual('3.0.5.0f5b2cf', versions_links.version(5))
        self.assertRaises(Exception, versions_links.version, 6)
        self.assertEqual('30.0.7.0f5b2cf', versions_links.version(7))
        self.assertRaises(Exception, versions_links.version, 8)
        self.assertEqual('3.0.10.0f5b2cf', versions_links.version(10))

    def test_download(self):
        Nexus().download('thirdparty', Gav('kaptcha', 'com.google.code', '2.3.2'), 'tmpfile.jar')
        self.assertTrue(os.path.exists('tmpfile.jar'))
        self.assertEqual(431542, os.stat('tmpfile.jar').st_size)
        os.remove('tmpfile.jar')

    def test_download_jcommune(self):
        gav, filename = JtalksArtifacts().download_war('jcommune', 2)
        self.assertTrue(os.path.exists(filename))
        self.assertEqual(38996276, os.stat(filename).st_size)
        os.remove(filename)

    def test_download_jc_plugin(self):
        files = JtalksArtifacts().download_plugins('jcommune', '3.0.6.8629f39', ['questions-n-answers-plugin'])
        self.assertTrue(os.path.exists(files[0]))
        self.assertEqual(861914, os.stat(files[0]).st_size)
        for filename in files:
            os.remove(filename)

    def test_deploy_plugins(self):
        os.mkdir('NexusTestDir/plugins')
        file('NexusTestDir/plugin.jar', 'w')
        JtalksArtifacts().deploy_plugins('NexusTestDir/plugins', ['NexusTestDir/plugin.jar'])

    def test_deploy_plugins_must_create_target_dir_if_it_not_exists(self):
        file('NexusTestDir/plugin.jar', 'w')
        JtalksArtifacts().deploy_plugins('NexusTestDir/notexisting', ['NexusTestDir/plugin.jar'])
        self.assertTrue(os.path.exists('NexusTestDir/notexisting/plugin.jar'))



import urllib
import sgmllib
import os
from Nexus import BuildNotFoundException

from jtalks.parser.PomFile import PomFile
from jtalks.util.Logger import Logger


class Nexus:
    """
    A class to work with Nexus (upload, download, search). Note, that for all the operations we need a build number
    to be passed into the constructor. Other properties may or may not be required (some of them can be determined,
    like project name and version by pom.xml).
    """
    base_url = "http://repo.jtalks.org/content/repositories/deployment-pipeline/"
    logger = Logger("OldNexus")

    def __init__(self, build_number):
        self.build_number = build_number

    def upload_war(self, pom_file_location):
        """
          Uploads a war to the Nexus. The path to pom.xml is acceptaced as an argument, by this path
          we also determean where war file is placed.
        """
        pom = PomFile(pom_file_location)
        artifact_version = pom.version()
        artifact_id = pom.artifact_id()

        maven_deploy_command = ("mvn deploy:deploy-file -Durl={2} " +
                                "-DrepositoryId=deployment-pipeline -DgroupId=deployment-pipeline -DartifactId={0} -Dpackaging=war " +
                                "-Dfile={0}-view/{0}-web-view/target/{0}.war -Dversion={1}"
        ).format(artifact_id, artifact_version, self.base_url)
        print maven_deploy_command

        return_code = os.system(maven_deploy_command)
        if (return_code != 0):
            self.logger.error("Maven returned error code: " + str(return_code))
            raise Exception("Maven returned error code: " + str(return_code))

    def download_war(self, project):
        self.logger.info("Looking up build #{0} for {1} project", self.build_number, project)
        war_url = self.get_war_url(project, self.build_number)
        self.logger.info("Downloading artifact: [{0}]", war_url)
        urllib.urlretrieve(war_url, project + ".war")
        return project + '.war'

    def get_war_url(self, project, build_number):
        group_id = "deployment-pipeline/"
        artifact_version_url = NexusPageWithVersions().parse(self.base_url + group_id + project).version(build_number)
        # get version by URL (last part is something like /jcommune/12.3.123/)
        artifact_version = artifact_version_url.rpartition(project + "/")[2].replace("/", "")
        return artifact_version_url + '{0}-{1}.war'.format(project, artifact_version)


class NexusPageWithVersions(sgmllib.SGMLParser):
    hyperlinks = []

    def parse(self, url_with_versions):
        feed = urllib.urlopen(url_with_versions)
        page_content = feed.read()
        feed.close()

        self.feed(page_content)
        self.close()
        return self

    def start_a(self, attributes):
        for name, link in attributes:
            if name == "href":
                self.hyperlinks.append(link)

    def version(self, build_number):
        for link in self.hyperlinks:
            # project-x.y.BUILD_NUMBER - old format, project-x.y.BUILD_NUMBER.git_hash - new format
            if ".{0}/".format(build_number) in link or \
                            ".{0}.".format(build_number) in link: return link
        raise BuildNotFoundException(
            "Couldn't find a build number {0}. Here are all the links: {1}".format(build_number, self.hyperlinks))



import os
import sys
import traceback

from ApplicationContext import ApplicationContext
from jtalks import __version__
from jtalks.ScriptSettings import ScriptSettings
from jtalks.util.Logger import Logger
from util.LibVersion import LibVersion


class Main:
    def __init__(self):
        self.logger = Logger('Main')

    def main(self, args, options):
        command = args[0]
        if command == 'version':
            print __version__
            exit(0)
        script_settings = ScriptSettings(options)
        app_context = ApplicationContext(script_settings)
        if script_settings.grab_envs == "true":
            app_context.environment_config_grabber().grab_jtalks_configs()
            # recreating them after configs were updated
            script_settings = ScriptSettings(options)
            app_context = ApplicationContext(script_settings)
        try:
            if command == 'deploy':
                LibVersion().log_lib_versions()
                app_context.deploy_command().deploy(script_settings.project, script_settings.build,
                                                    script_settings.get_app_final_name(),
                                                    script_settings.get_plugins())
            elif command == "upload-to-nexus":
                LibVersion().log_lib_versions()
                app_context.old_nexus().upload_war('pom.xml')
            elif command == "list-envs":
                app_context.env_list().list_envs()
            elif command == 'load-db-from-backup':
                LibVersion().log_lib_versions()
                app_context.load_db_from_backup().load()
            else:
                error = 'Command was not recognized, you can use: deploy, list-envs, load-db-from-backup. ' \
                        'Also see jtalks -h'
                self.logger.error(error)
                raise RuntimeError(error)
        except:
            self.logger.error("Program finished with errors")
            if options.debug:
                print("Root cause: %s" % traceback.format_exc())
                sys.exit(1)



#!/usr/bin/env python
from distutils.core import setup

from setuptools import find_packages

from jtalks import __version__

setup(name='jtalks-cicd',
      version=__version__,
      description='Installs JTalks apps like jcommune and poulpe configuring them. Usually used by CI to implement CD '
                  '(continuous delivery)',
      author='Stanislav Bashkyrtsev',
      author_email='stanislav.bashkirtsev@gmail.com',
      url='http://github.com/jtalks-org/jtalks-cicd',
      install_requires=['distribute==0.7.3', 'requests', 'GitPython', 'paramiko', 'mock', 'ConfigParser',
                        'MySQL-Python', 'mysql-connector-python'],
      tests_require=['mock'],
      scripts=['bin/jtalks'],
      packages=find_packages(),
      test_suite='tests',
      # after package installation we'll have a nice directory instead of zipped artifact which is
      # easier to work with
      zip_safe=False
      #include_package_data=True should add MANIFEST.in first to include non-py files if needed
)



from datetime import datetime


class Logger:
    class_name = None

    def __init__(self, class_name):
        self.class_name = class_name

    def info(self, message, *args):
        self.__log__(colors.OKGREEN, "INFO", message, args)

    def warn(self, message, *args):
        self.__log__(colors.WARNING, "WARN", message, args)

    def error(self, message, *args):
        self.__log__(colors.FAIL, "ERROR", message, args)

    def __log__(self, color, level, message, args):
        print(color + "[{0}][JTALKS][{1}][{2}] ".format(datetime.now().isoformat(), level, self.class_name) +
              message.format(*args) + colors.END)


class colors:
    HEADER = '\033[35m'
    OKBLUE = '\033[34m'
    OKGREEN = '\033[32m'
    WARNING = '\033[33m'
    FAIL = '\033[31m'
    END = '\033[0m'

    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''



from datetime import datetime
import os
import shutil

from jtalks.util.Logger import Logger


class Backuper:
    """
      Responsible for backing up artifacts like war files, config files, as well as DB backups.
      Stores backups in the folder and allows to restore those backups from it.
    """
    logger = Logger("Backuper")
    BACKUP_FOLDER_DATE_FORMAT = "%Y_%m_%dT%H_%M_%S"

    def __init__(self, root_backup_folder, db_operations):
        """
        :param str root_backup_folder: a directory to put backups to, it may contain backups from different envs,
            thus a folder for each env will be created
        :param jtalks.db.DbOperations.DbOperations db_operations: instance of DbOperations
        """
        self.backup_folder = self.create_folder_to_backup(root_backup_folder)
        self.db_operations = db_operations

    def create_folder_to_backup(self, backup_folder):
        """
          We don't just put backups into a backup folder, we're creating a new folder there
          with current date so that our previous backups are kept there. E.g.:
          '/var/backups/prod/20130302T195924'
          :param str backup_folder: the root folder to create folders with datetime names for each backup
        """
        now = datetime.now().strftime(self.BACKUP_FOLDER_DATE_FORMAT)
        final_backup_folder = os.path.join(backup_folder, now)
        if not os.path.exists(final_backup_folder):
            self.logger.info("Creating a folder to store back ups: [{0}]", final_backup_folder)
            os.makedirs(final_backup_folder)
        return final_backup_folder

    def back_up_dir(self, folder_path):
        if os.path.exists(folder_path):
            head, folder_name = os.path.split(folder_path)
            backup_dst = os.path.join(self.backup_folder, folder_name)
            self.logger.info('Backing up [{0}] to [{1}]', folder_path, backup_dst)
            shutil.copytree(folder_path, backup_dst)
        else:
            backup_dst = None
            self.logger.info("There was no previous folder [{0}], so nothing to backup", folder_path)
        return backup_dst

    def back_up_db(self):
        self.db_operations.backup_database(self.backup_folder)



class LoadDbFromBackup:
    def __init__(self, db, ssh):
        """
        :param jtalks.DB.DB db: db operations
        @param ssh:
        @return:
        """
        self.db = db
        self.ssh = ssh

    def load(self):
        self.ssh.download_backup_prod_db()
        self.db.connect_to_database()
        self.db.restore_database_from_file(self.ssh.sftpBackupFileName)
        self.db.update_properties_to_preprod()
        self.db.connection.close()
        self.ssh.remove_backup_prod_db_file()



import os
import ConfigParser

import paramiko
from jtalks.util.Logger import Logger


class SSH:
    env = None
    config = None
    sftpTransport = None
    sftpClient = None
    sftpHost = None
    sftpPort = None
    sftpUser = None
    sftpPass = None
    sftpBackupArchive = None
    sftpBackupFileName = None
    logger = Logger("SSH")

    def __init__(self, script_settings):
        self.env = script_settings.env
        self.config = ConfigParser.ConfigParser()
        self.config.read(script_settings.get_env_configs_dir() + self.env + "/ssh.cfg")
        self.sftpHost = self.config.get('sftp', 'sftp_host')
        self.sftpPort = self.config.get('sftp', 'sftp_port')
        self.sftpUser = self.config.get('sftp', 'sftp_user')
        self.sftpPass = self.config.get('sftp', 'sftp_pass')
        self.sftpBackupArchive = self.config.get('sftp', 'sftp_backup_archive')
        self.sftpBackupFileName = self.config.get('sftp', 'sftp_backup_filename')

    def download_backup_prod_db(self):
        self.sftp_connection()
        attrList = self.sftpClient.listdir_attr('.')
        attrTimeTmp = 0
        dirWithLastUpdate = None
        for attr in attrList:
            if attrTimeTmp < attr.st_mtime:
                attrTimeTmp = attr.st_mtime
                dirWithLastUpdate = attr.filename
        fileFromFtp = dirWithLastUpdate + '/' + self.sftpBackupArchive
        self.logger.info('Downloading file from ftp [{0}]', fileFromFtp)
        self.sftpClient.get(fileFromFtp, './' + self.sftpBackupArchive)
        os.popen('bunzip2 -d ' + self.sftpBackupArchive)
        self.sftp_connection_close()

    def sftp_connection_close(self):
        self.sftpClient.close()
        self.sftpTransport.close()

    def sftp_connection(self):
        self.logger.info("Getting a DB backup from {0}", self.sftpHost)
        self.sftpTransport = paramiko.Transport((self.sftpHost, int(self.sftpPort)))
        self.sftpTransport.connect(username=self.sftpUser, password=self.sftpPass)
        self.sftpClient = paramiko.SFTPClient.from_transport(self.sftpTransport)

    def remove_backup_prod_db_file(self):
        os.popen('rm -rf ' + self.sftpBackupFileName)

  



from Nexus import BuildNotFoundException
from jtalks.util.Logger import Logger


class DeployCommand:
    NEXUS_URL = 'http://repo.jtalks.org/content/repositories/deployment-pipeline/deployment-pipeline/'
    logger = Logger('DeployCommand')

    def __init__(self, jtalks_artifacts, old_nexus, tomcat, sanity_test, backuper, scriptsettings):
        """
        :param jtalks.ScriptSettings.ScriptSettings scriptsettings: config files to be deployed along with the app
        :param jtalks.Nexus.JtalksArtifacts jtalks_artifacts: downloads JTalks artifacts from Nexus
        :param jtalks.OldNexus.Nexus old_nexus: used to download artifacts if it's old and is kept in old repo
        :param jtalks.Tomcat.Tomcat tomcat: manages tomcat
        :param jtalks.sanity.SanityTest.SanityTest sanity_test: runs the tests after an app is deployed
        :param jtalks.backup.Backuper.Backuper backuper: backs up DB, tomcat
        """
        self.scriptsettings = scriptsettings
        self.tomcat = tomcat
        self.jtalks_artifacts = jtalks_artifacts
        self.sanity_test = sanity_test
        self.old_nexus = old_nexus
        self.backuper = backuper

    def deploy(self, project, build, app_final_name, plugins=[]):
        self.__validate_params_and_raise__(project, build)
        plugin_files = []
        try:
            gav, filename = self.jtalks_artifacts.download_war(project, build)
            plugin_files = self.jtalks_artifacts.download_plugins(project, gav.version, plugins)
        except BuildNotFoundException:
            filename = self.old_nexus.download_war(project)
        self.tomcat.stop()
        self.backuper.back_up_dir(self.tomcat.tomcat_location)
        self.backuper.back_up_db()
        self.tomcat.move_to_webapps(filename, app_final_name)
        self.jtalks_artifacts.deploy_plugins(self.scriptsettings.get_plugins_dir(), plugin_files)
        self.scriptsettings.deploy_configs()
        self.tomcat.start()
        self.sanity_test.check_app_started_correctly()

    def __validate_params_and_raise__(self, project, build):
        if build is None:
            self.logger.error('Build number was not specified, see [{0}] to get list of builds', self.NEXUS_URL)
            raise RuntimeError
        if project not in ['jcommune', 'poulpe', 'antarcticle']:
            self.logger.error('A correct project should be specified: [poulpe, jcommune, antarcticle]. Actual: [{0}]',
                              project)
            raise RuntimeError
